@@ -7,6 +7,7 @@ from urllib.parse import quote, unquote
 
 # Pip
 from ksimpleapi import Api, Request
+from kstopit import signal_timeoutable
 
 # Local
 from .models import SearchResultListing, Listing, Region
@@ -45,7 +46,7 @@ class Zillow(Api):
         except Exception as e:
             if debug:
                 print(e)
-            
+
             return None
 
     def get_listings(
@@ -150,6 +151,7 @@ class Zillow(Api):
 
             return None
 
+    @signal_timeoutable(name='zillow.download')
     def download_listing_assets(
         self,
         listing: Listing,
@@ -161,50 +163,33 @@ class Zillow(Api):
         sleep_s_between_failed_requests: Optional[float] = None,
         debug: Optional[bool] = None,
         download_images_max_concurent_processes: Optional[int] = None,
-        create_video_max_concurent_processes: Optional[int] = None
+        create_video_max_concurent_processes: Optional[int] = None,
+        timeout: Optional[int]= 10*60
     ) -> Tuple[Optional[str], List[str]]:
         from simple_multiprocessing import MultiProcess, Task
         """Returns:
             Tuple[Optional[str], Optional[str], Optional[List[str]]]: [video_path, main_image_path, image_paths]
         """
 
-        def create_video(max_concurent_processes: Optional[int] = None):
-            if listing.video:
-                video_path = os.path.join(out_folder_path, 'video.mp4')
-                video_temp_folder_path = os.path.join(out_folder_path, 'video_temp')
-                os.makedirs(video_temp_folder_path, exist_ok=True)
-                video_res = self.__download_listing_video(
-                    listing.video.url_m3u8,
-                    video_temp_folder_path,
-                    video_path,
-                    user_agent=user_agent,
-                    proxy=proxy,
-                    max_request_try_count=max_request_try_count,
-                    sleep_s_between_failed_requests=sleep_s_between_failed_requests,
-                    debug=debug,
-                    max_concurent_processes=max_concurent_processes
-                )
-                shutil.rmtree(video_temp_folder_path)
-
-                return video_path if video_res else None
-            else:
-                return None
-
-        return MultiProcess([
-            Task(create_video, max_concurent_processes=create_video_max_concurent_processes),
-            Task(
-                self.__download_listing_images,
-                [l.url for l in listing.images],
-                out_folder_path,
-                image_pre_name,
+        if listing.video:
+            video_path = os.path.join(out_folder_path, 'video.mp4')
+            video_temp_folder_path = os.path.join(out_folder_path, 'video_temp')
+            os.makedirs(video_temp_folder_path, exist_ok=True)
+            video_res = self.__download_listing_video(
+                listing.video.url_m3u8,
+                video_temp_folder_path,
+                video_path,
                 user_agent=user_agent,
                 proxy=proxy,
                 max_request_try_count=max_request_try_count,
                 sleep_s_between_failed_requests=sleep_s_between_failed_requests,
                 debug=debug,
-                max_concurent_processes=download_images_max_concurent_processes
             )
-        ]).solve()
+            shutil.rmtree(video_temp_folder_path)
+
+            return video_path if video_res else None
+        else:
+            return None
 
     # ------------------------------------------------------- Private methods -------------------------------------------------------- #
 
@@ -228,7 +213,7 @@ class Zillow(Api):
                 print('err', e)
 
             return None
-    
+
     def __get_region_bounds(
         self,
         region_str: str,
@@ -244,7 +229,7 @@ class Zillow(Api):
                 print(e)
 
             return None
-    
+
     def __download_listing_video(
         self,
         url: str,
@@ -268,19 +253,19 @@ class Zillow(Api):
         playlist = info.playlists[-1]
         playlist_url = playlist.base_uri+playlist.uri
         playlist = m3u8.load(playlist_url)
-        urls_paths = {s.base_uri+s.uri:os.path.join(temp_folder_path, s.uri) for s in playlist.segments}
-        res = self.download_async(urls_paths, user_agent=user_agent, proxy=proxy, max_request_try_count=max_request_try_count, debug=debug, max_concurent_processes=max_concurent_processes)
-        paths = list(urls_paths.values())
+        file_paths = []
 
-        if False in res:
-            [os.remove(p) for p in paths if os.path.exists(p)]
+        for s in playlist.segments:
+            url_download_path = s.base_uri+s.uri
+            file_path = os.path.join(temp_folder_path, s.uri)
 
-            return False
+            self.download(url_download_path, file_path, user_agent=user_agent, proxy=proxy, max_request_try_count=max_request_try_count, debug=debug)
+            file_paths.append(file_path)
 
         p_ts = os.path.join(temp_folder_path, 'temp.ts')
 
-        if not ffmpeg.concat_videos_copy(paths, p_ts, debug=debug):
-            [os.remove(p) for p in paths if os.path.exists(p)]
+        if not ffmpeg.concat_videos_copy(file_paths, p_ts, debug=debug):
+            [os.remove(p) for p in file_paths if os.path.exists(p)]
 
             return False
 
